import { Form, Input, message, Modal, Row, Select } from 'antd';
import { history, useDispatch, useIntl, useSelector } from 'umi';
import styles from '../../styles.module.less';
import ImageUpload from '@/components/ImageUpload';
import {
  CallBackLinks,
  IDataCenter,
  IDataCenterState,
  IEditProduct,
  IEditProductState,
  OnImageUpload,
} from '@/typings';
import Buttons from '@/pages/Product/ProductEdit/Steps/Buttons';
import { errorHandler } from '@/utils/errorHandler';
import { saveStateAction } from '@/pages/Product/ProductEdit/models';
import { useSafeState, useUpdateEffect } from 'ahooks';
import React from 'react';
import { setLinkCallBack } from '@/models/linkCallbacks';

export default function () {
  const intl = useIntl();
  const [form] = Form.useForm();
  const dispatch = useDispatch();

  const editProduct = useSelector<IEditProductState, IEditProduct>((state) => {
    return state.editProduct;
  });

  const dataCenter = useSelector<IDataCenterState, IDataCenter>(
    (state) => state.dataCenter,
  );

  const [disable, setDisable] = useSafeState({ save: true, nextStep: true });

  const validateFields = async () => {
    try {
      await form.validateFields();
      return true;
    } catch (error) {
      return false;
    }
  };

  const onSubmit = async (advanceStep = false) => {
    try {
      const valid = await validateFields();
      if (!valid) return;
      const values = form.getFieldsValue();
      values.configurableSku = values.configurableSku.trim();
      return await saveStateAction(
        dispatch,
        editProduct,
        { generalDetails: values },
        editProduct.current + (advanceStep ? 1 : 0),
        !disable.save,
      );
    } catch (error) {
      errorHandler(error);
      return false;
    }
  };

  const onRedirectLink = async () => {
    if (disable.save && !disable.nextStep) return true;
    const valid = await validateFields();
    const promise = new Promise((resolve) => {
      if (!valid)
        Modal.confirm({
          title: intl.formatMessage({
            id: 'product.notCompletedPopup',
          }),
          onOk: () => resolve(true),
          onCancel: () => resolve(false),
          cancelText: intl.formatMessage({ id: 'product.cancel' }),
          okText: intl.formatMessage({ id: 'product.quit' }),
        });
      if (valid)
        Modal.confirm({
          title: intl.formatMessage({ id: 'product.wouldYouLikeToSaveChanges?' }),
          onOk: async () => {
            const success = await onSubmit();
            const isNewProduct = editProduct.isNew;
            const messageText = isNewProduct
              ? intl.formatMessage({ id: 'product.theProductDraftHasBeenSaved' })
              : intl.formatMessage({ id: 'product.productHasBeenSaved' });
            if (success) message.success(messageText);
            resolve(success);
          },
          onCancel: () => resolve(true),
          cancelText: intl.formatMessage({ id: 'product.discard' }),
          okText: intl.formatMessage({ id: 'product.save' }),
        });
    });
    return promise;
  };

  const checkIfAllFillIn = (values: any, allValues: any, firstTimeCheck = false) => {
    const requiredFields = [
      'productName',
      'configurableSku',
      'countryOfOrigin',
      'itemCategory',
    ];
    const requiredFieldsOnly = Object.entries(allValues).filter(([key, value]) => {
      if (!requiredFields.includes(key)) return false;
      if (!value) return false;
      return true;
    });

    if (firstTimeCheck && requiredFieldsOnly.length === 4)
      return setDisable({ save: true, nextStep: false });

    if (requiredFieldsOnly.length !== 4)
      return setDisable({ save: true, nextStep: true });

    setDisable({ save: false, nextStep: false });
  };

  const resetGoBackCallBack = () => {
    const regex = /^\/home\/product\/edit-product\//;
    const isEdit = regex.test(history.location.pathname);
    const callBackLink: CallBackLinks = isEdit
      ? '/home/product/edit-product/:id'
      : '/home/product/create-product';
    dispatch({
      type: setLinkCallBack,
      payload: {
        [callBackLink]: () => onRedirectLink(), // doing this due to an issue that disable value is not updated in the callback
      },
    });
  };

  React.useEffect(() => {
    checkIfAllFillIn(null, editProduct.data.generalDetails || {}, true);
  }, [editProduct.loading]);

  // need to do this weird update because disable is not being updated properly in model
  React.useEffect(() => {
    resetGoBackCallBack();
  }, [disable]);

  useUpdateEffect(() => {
    if (editProduct.data.generalDetails === null) form.resetFields();
    else form.setFieldsValue({ ...editProduct.data.generalDetails });
  }, [editProduct]);

  const onUpload: OnImageUpload = (fileKey) => {
    form.setFieldsValue({ productImage: fileKey });
  };

  const onNext = async () => {
    const result = (await onSubmit(true)) as boolean;
    if (!result) setDisable({ nextStep: true, save: true });
    return result;
  };

  return (
    <Row className={styles.stepContainer}>
      <Form
        onValuesChange={checkIfAllFillIn}
        initialValues={{ ...editProduct.data.generalDetails }}
        form={form}
        className={styles.form}
        labelAlign={'left'}
      >
        <Form.Item
          label={intl.formatMessage({ id: 'product.productName' })}
          name={'productName'}
          rules={[{ required: true }]}
        >
          <Input />
        </Form.Item>
        <Form.Item
          label={intl.formatMessage({ id: 'product.configurableSku' })}
          name={'configurableSku'}
          rules={[
            {
              required: true,
              validator: (_, value) => {
                if (!value)
                  return Promise.reject(
                    new Error(
                      `${intl.formatMessage({
                        id: 'product.pleaseEnter',
                      })} ${intl.formatMessage({ id: 'product.configurableSku' })}`,
                    ),
                  );
                const trim = value.trim();
                const regex = /^[A-Za-z-0-9]{1,64}$/;
                const valid = regex.test(trim);
                if (!valid)
                  return Promise.reject(
                    new Error(
                      intl.formatMessage({
                        id: 'product.skuCanOnlyContainLettersNumbersAndDashWithMax64Characters',
                      }),
                    ),
                  );

                return Promise.resolve();
              },
            },
          ]}
        >
          <Input disabled={!editProduct.isNew} />
        </Form.Item>
        <Form.Item
          label={intl.formatMessage({ id: 'product.factory' })}
          name={'factory'}
          rules={[{ required: false }]}
        >
          <Input />
        </Form.Item>
        <Form.Item
          label={intl.formatMessage({ id: 'product.description' })}
          name={'description'}
          rules={[{ required: false }]}
        >
          <Input.TextArea />
        </Form.Item>
        <Form.Item
          label={intl.formatMessage({ id: 'product.countryOfOrigin' })}
          name={'countryOfOrigin'}
          rules={[{ required: true }]}
        >
          <Select>
            {dataCenter!.countries!.map((e) => {
              return <Select.Option key={e.custom1}>{e.lookupCode}</Select.Option>;
            })}
            <Select.Option key={'US'}>US</Select.Option>
          </Select>
        </Form.Item>
        <Form.Item
          label={intl.formatMessage({ id: 'product.otherNames' })}
          name={'otherNames'}
          rules={[{ required: false }]}
        >
          <Input />
        </Form.Item>
        <Form.Item
          label={intl.formatMessage({ id: 'product.productImage' })}
          name={'productImage'}
          rules={[{ required: false }]}
        >
          <ImageUpload onUpload={onUpload} />
        </Form.Item>
        <Form.Item
          label={intl.formatMessage({ id: 'product.itemCategory' })}
          name={'itemCategory'}
          rules={[{ required: true }]}
        >
          <Select>
            <Select.Option key={'food'}>Food</Select.Option>
          </Select>
        </Form.Item>
      </Form>
      <Buttons
        productId={editProduct.productId}
        disable={disable}
        step={editProduct.current}
        onNextStep={onNext}
        isNewProduct={editProduct.isNew}
        onSave={async () => {
          const success = await onSubmit();
          if (success) setDisable({ save: true, nextStep: false });
          return success as boolean;
        }}
      />
    </Row>
  );
}
